# © Copyright Databand.ai, an IBM Company 2022

__version__ = "0.1.0"

__title__ = "dbnd-examples"
__description__ = "databand.ai example project"
__url__ = "http://www.databand.ai/"
__uri__ = __url__
__doc__ = __description__ + " <" + __uri__ + ">"

__author__ = "Evgeny Shulman"
__email__ = "evgeny.shulman@databand.ai"

__license__ = "Commercial Licence"
__copyright__ = "Copyright (c) 2018 databand.ai"
