# © Copyright Databand.ai, an IBM Company 2022

from dbnd_examples_orchestration.orchestration.tool_mxnet import (
    mxnet_digit_mnist,
    mxnet_fashion_mnist,
)


__all__ = ["mxnet_digit_mnist", "mxnet_fashion_mnist"]
