# © Copyright Databand.ai, an IBM Company 2022


class SagemakerData(object):
    amazon_reviews_raw_data = "s3://amazon-reviews-pds/tsv/amazon_reviews_us_Digital_Video_Download_v1_00.tsv.gz"


sagemaker_data_repo = SagemakerData()
